@@ -4,7 +4,7 @@ from keras_cv_attention_models import model_surgery
 from keras_cv_attention_models import efficientnet
 from keras_cv_attention_models.attention_layers import activation_by_name, add_pre_post_process
 from keras_cv_attention_models.download_and_load import reload_model_weights
-from keras_cv_attention_models.coco.eval_func import DecodePredictions
+from keras_cv_attention_models.coco import eval_func, anchors_func
 
 BATCH_NORM_EPSILON = 1e-3
 PRETRAINED_DICT = {
@@ -141,10 +141,9 @@ def EfficientDet(
     head_depth=3,
     num_channels=64,
     use_weighted_sum=True,
-    use_anchor_free_mode=False,
-    use_yolor_anchors_mode=False,
-    use_object_scores="auto",  # "auto" means same with use_anchor_free_mode
-    num_anchors="auto",  # "auto" means 1 if use_anchor_free_mode else 9
+    anchors_mode="efficientdet",
+    use_object_scores="auto",  # "auto" means: True if anchors_mode=="anchor_free" or anchors_mode=="yolor", else False
+    num_anchors="auto",  # "auto" means: anchors_mode=="anchor_free" -> 1, anchors_mode=="yolor" -> 3, else 9
     num_classes=90,
     use_sep_conv=True,
     activation="swish",
@@ -153,15 +152,13 @@ def EfficientDet(
     pretrained="coco",
     model_name=None,
     pyramid_levels_min=3,  # Init anchors for model prediction, not for model structure
-    anchor_scale="auto",  # Init anchors for model prediction. "auto" means 1 if use_anchor_free_mode else 4
+    anchor_scale="auto",  # Init anchors for model prediction. "auto" means 1 if (anchors_mode=="anchor_free" or anchors_mode=="yolor"), else 4
     rescale_mode="torch",  # Model precessing input, not for model structure
     input_shape=None,  # Not using, recieving parameter
     kwargs=None,  # Not using, recieving parameter
 ):
-    if freeze_backbone:
-        backbone.trainable = False
-    else:
-        backbone.trainable = True
+    backbone.trainable = False if freeze_backbone else True
+    use_object_scores, num_anchors, anchor_scale = anchors_func.get_anchors_mode_parameters(anchors_mode, use_object_scores, num_anchors, anchor_scale)
 
     if isinstance(features_pick[0], str):
         fpn_features = [backbone.get_layer(layer_name) for layer_name in features_pick]
@@ -183,10 +180,6 @@ def EfficientDet(
         fpn_features = bi_fpn(fpn_features, num_channels, use_weighted_sum, use_sep_conv, activation=activation, name="biFPN_{}_".format(id + 1))
 
     # Outputs
-    use_object_scores = (use_yolor_anchors_mode or use_anchor_free_mode) if use_object_scores == "auto" else use_object_scores
-    if num_anchors == "auto":
-        num_anchors = 1 if use_anchor_free_mode else (3 if use_yolor_anchors_mode else 9)
-
     bboxes_features = det_header_pre(fpn_features, num_channels, head_depth, use_sep_conv, activation=activation, name="regressor_")
     bboxes_out = det_header_post(bboxes_features, 4, num_anchors, bias_init="zeros", use_sep_conv=use_sep_conv, head_activation=None, name="regressor_")
     if use_object_scores:
@@ -209,8 +202,7 @@ def EfficientDet(
     # For prediction
     # AA = {"aspect_ratios": anchor_aspect_ratios, "num_scales": anchor_num_scales, "anchor_scale": anchor_scale, "grid_zero_start": anchor_grid_zero_start}
     pyramid_levels = [pyramid_levels_min, pyramid_levels_min + len(features_pick) + additional_features - 1]  # -> [3, 7]
-    anchor_scale = (1 if use_anchor_free_mode or use_yolor_anchors_mode else 4) if anchor_scale == "auto" else anchor_scale
-    post_process = DecodePredictions(backbone.input_shape[1:], pyramid_levels, anchor_scale, use_anchor_free_mode, use_yolor_anchors_mode, use_object_scores)
+    post_process = eval_func.DecodePredictions(backbone.input_shape[1:], pyramid_levels, anchors_mode, use_object_scores, anchor_scale)
     add_pre_post_process(model, rescale_mode=rescale_mode, post_process=post_process)
     # model.backbone = backbone
     return model
@@ -276,7 +268,7 @@ def EfficientDetD7(input_shape=(1536, 1536, 3), freeze_backbone=False, num_class
     if backbone is None:
         backbone = efficientnet.EfficientNetV1B6(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation)
         features_pick = ["stack_2_block5_output", "stack_4_block7_output", "stack_6_block2_output"]
-    anchor_scale = kwargs.pop("anchor_scale", 1 if kwargs.get("use_anchor_free_mode", False) else 5)
+    anchor_scale = kwargs.pop("anchor_scale", 5)
     model_name = kwargs.pop("model_name", "efficientdet_d7")
     return EfficientDet(**locals(), fpn_depth=8, head_depth=5, num_channels=384, use_weighted_sum=False, **kwargs)
 
@@ -298,7 +290,7 @@ def EfficientDetLite0(input_shape=(320, 320, 3), freeze_backbone=False, num_clas
         backbone = efficientnet.EfficientNetV1Lite0(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation)
         features_pick = ["stack_2_block1_output", "stack_4_block2_output", "stack_6_block0_output"]
     model_name = kwargs.pop("model_name", "efficientdet_lite0")
-    anchor_scale = kwargs.pop("anchor_scale", 1 if kwargs.get("use_anchor_free_mode", False) else 3)
+    anchor_scale = kwargs.pop("anchor_scale", 3)
     return EfficientDet(**locals(), fpn_depth=3, head_depth=3, num_channels=64, use_weighted_sum=False, rescale_mode="tf", **kwargs)
 
 
@@ -307,7 +299,7 @@ def EfficientDetLite1(input_shape=(384, 384, 3), freeze_backbone=False, num_clas
         backbone = efficientnet.EfficientNetV1Lite1(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation)
         features_pick = ["stack_2_block2_output", "stack_4_block3_output", "stack_6_block0_output"]
     model_name = kwargs.pop("model_name", "efficientdet_lite1")
-    anchor_scale = kwargs.pop("anchor_scale", 1 if kwargs.get("use_anchor_free_mode", False) else 3)
+    anchor_scale = kwargs.pop("anchor_scale", 3)
     return EfficientDet(**locals(), fpn_depth=4, head_depth=3, num_channels=88, use_weighted_sum=False, rescale_mode="tf", **kwargs)
 
 
@@ -316,7 +308,7 @@ def EfficientDetLite2(input_shape=(448, 448, 3), freeze_backbone=False, num_clas
         backbone = efficientnet.EfficientNetV1Lite2(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation)
         features_pick = ["stack_2_block2_output", "stack_4_block3_output", "stack_6_block0_output"]
     model_name = kwargs.pop("model_name", "efficientdet_lite2")
-    anchor_scale = kwargs.pop("anchor_scale", 1 if kwargs.get("use_anchor_free_mode", False) else 3)
+    anchor_scale = kwargs.pop("anchor_scale", 3)
     return EfficientDet(**locals(), fpn_depth=5, head_depth=3, num_channels=112, use_weighted_sum=False, rescale_mode="tf", **kwargs)
 
 
@@ -325,7 +317,7 @@ def EfficientDetLite3(input_shape=(512, 512, 3), freeze_backbone=False, num_clas
         backbone = efficientnet.EfficientNetV1Lite3(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation)
         features_pick = ["stack_2_block2_output", "stack_4_block4_output", "stack_6_block0_output"]
     model_name = kwargs.pop("model_name", "efficientdet_lite3")
-    anchor_scale = kwargs.pop("anchor_scale", 1 if kwargs.get("use_anchor_free_mode", False) else 4)
+    anchor_scale = kwargs.pop("anchor_scale", 4)
     return EfficientDet(**locals(), fpn_depth=6, head_depth=4, num_channels=160, use_weighted_sum=False, rescale_mode="tf", **kwargs)
 
 
@@ -334,7 +326,7 @@ def EfficientDetLite3X(input_shape=(640, 640, 3), freeze_backbone=False, num_cla
         backbone = efficientnet.EfficientNetV1Lite3(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation)
         features_pick = ["stack_2_block2_output", "stack_4_block4_output", "stack_6_block0_output"]
     model_name = kwargs.pop("model_name", "efficientdet_lite3x")
-    anchor_scale = kwargs.pop("anchor_scale", 1 if kwargs.get("use_anchor_free_mode", False) else 3)
+    anchor_scale = kwargs.pop("anchor_scale", 3)
     return EfficientDet(**locals(), fpn_depth=6, head_depth=4, num_channels=200, use_weighted_sum=False, rescale_mode="tf", **kwargs)
 
 
@@ -343,5 +335,5 @@ def EfficientDetLite4(input_shape=(640, 640, 3), freeze_backbone=False, num_clas
         backbone = efficientnet.EfficientNetV1Lite4(input_shape=input_shape, num_classes=0, output_conv_filter=0, activation=activation)
         features_pick = ["stack_2_block3_output", "stack_4_block5_output", "stack_6_block0_output"]
     model_name = kwargs.pop("model_name", "efficientdet_lite4")
-    anchor_scale = kwargs.pop("anchor_scale", 1 if kwargs.get("use_anchor_free_mode", False) else 3)
+    anchor_scale = kwargs.pop("anchor_scale", 3)
     return EfficientDet(**locals(), fpn_depth=7, head_depth=4, num_channels=224, use_weighted_sum=False, rescale_mode="tf", **kwargs)
